package hu.uw.pallergabor.irblegw;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanResult;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.util.UUID;


public class MainActivity extends Activity {

    private static final String LOG_TAG = "irblegw.MainActivity";
    private static final long SCAN_PERIOD = 10*1000L;
    private static final long HEARTBEAT_RESPONSE_DELAY = 100L;
    private static final int REQUEST_ENABLE_BT = 1;

    // 0x2902 org.bluetooth.descriptor.gatt.client_characteristic_configuration.xml
    public final static UUID CLIENT_CHARACTERISTIC_CONFIGURATION_UUID = UUID.fromString("00002902-0000-1000-8000-00805F9B34FB");
    public static MainActivity thisActivity;
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothLeScanner scanner;
    private BluetoothDevice gattExplorerDevice;
    private GattExplorerFoundTask geFoundTask = new GattExplorerFoundTask();
    private GattExplorerErrorTask geErrorTask = new GattExplorerErrorTask();
    private GattExplorerConnectedTask geConnectedTask = new GattExplorerConnectedTask();
    private GattExplorerDisconnectedTask geDisconnectedTask = new GattExplorerDisconnectedTask();
    private GattExplorerDiscoveredTask geDiscoveredTask = new GattExplorerDiscoveredTask();
    private MyScanCallback myScanCallback = new MyScanCallback();
    private TextView connStatusTV;
    private Button actionButton;
    private IRGWServiceIF irgwserviceIF;
    private IRGWServiceConnection irgwServiceConnection = new IRGWServiceConnection();

    public enum ConnState {
        IDLE, SCANNING, CONNECTED
    }

    private ConnState connState = ConnState.IDLE;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        thisActivity = this;
        irgwserviceIF = null;
        setContentView(R.layout.activity_main);
        connStatusTV = (TextView) findViewById(R.id.connstatus);
        actionButton = (Button)findViewById(R.id.btn_action);

        if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
            Toast.makeText(this, R.string.ble_not_supported, Toast.LENGTH_SHORT).show();
            finish();
        }

        // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
        // BluetoothAdapter through BluetoothManager.
        final BluetoothManager bluetoothManager =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = bluetoothManager.getAdapter();

        // Checks if Bluetooth is supported on the device.
        if (bluetoothAdapter == null) {
            Toast.makeText(this, R.string.error_bluetooth_not_supported, Toast.LENGTH_SHORT).show();
            finish();
            return;
        }
        actionButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                switch (connState) {
                    case IDLE:
                        startLEScan();
                        break;

                    case SCANNING:
                        stopLEScan();
                        break;

                    case CONNECTED:
                        disconnect();
                        break;
                }
            }
        });
        Intent serviceIntent = new Intent(this,IRGWService.class);
        bindService( serviceIntent, irgwServiceConnection, Context.BIND_AUTO_CREATE);
        connState = ConnState.IDLE;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    public void eventGWDiscovered() {
        runOnUiThread(geDiscoveredTask);
    }

    public void eventGWConnected() {
        runOnUiThread(geConnectedTask);
        connState = ConnState.CONNECTED;
    }

    public void eventGWDisconnected() {
        runOnUiThread(geDisconnectedTask);
        connState = ConnState.IDLE;
    }

    public void eventGWKey(long v) {
        KeyReceivedTask kt = new KeyReceivedTask(v);
        runOnUiThread(kt);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // User chose not to enable Bluetooth.
        if (requestCode == REQUEST_ENABLE_BT && resultCode == Activity.RESULT_CANCELED) {
            finish();
            return;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private void startLEScan() {
        gattExplorerDevice = null;
        scanner  = bluetoothAdapter.getBluetoothLeScanner();
        scanner.startScan(myScanCallback);
        connStatusTV.setText(R.string.ge_scanning);
        connState = ConnState.SCANNING;
    }

    private void stopLEScan() {
        scanner.stopScan(myScanCallback);
        connState = ConnState.IDLE;
    }

    private void disconnect() {
        connState = ConnState.IDLE;
        if( irgwserviceIF != null )
            try {
                irgwserviceIF.disconnect();
            } catch (RemoteException e) {
                Toast.makeText(MainActivity.this,
                        MainActivity.this.getString( R.string.cant_call_service),
                        Toast.LENGTH_LONG).show();
                Log.e(LOG_TAG, "disconnect() invocation error", e);
            }
    }

    // Device scan callback.
    class MyScanCallback extends ScanCallback {
        public void onScanFailed(int errorCode) {
            MainActivity.this.runOnUiThread(geErrorTask);
            connState = ConnState.IDLE;
        }

        public void onScanResult(int callbackType, ScanResult result) {
            String deviceName = result.getDevice().getName();
            Log.d(LOG_TAG, "device found: '" + deviceName + "'");
            if( "irblegw".equals( deviceName ) ) {
                scanner.stopScan(myScanCallback);
                gattExplorerDevice = result.getDevice();
                MainActivity.this.runOnUiThread(geFoundTask);
                Intent serviceIntent = new Intent(MainActivity.this, IRGWService.class);
                serviceIntent.putExtra("deviceAddress",gattExplorerDevice.getAddress());
                startService(serviceIntent);
            }

        }
    }

    class GattExplorerFoundTask implements Runnable {
        public void run() {
            connStatusTV.setText(R.string.ge_found);
        }
    }

    class GattExplorerErrorTask implements Runnable {
        public void run() {
            connStatusTV.setText(R.string.ge_error);
        }

    }

    class GattExplorerConnectedTask implements Runnable {
        public void run() {
            connStatusTV.setText(R.string.ge_connected);
        }

    }

    class GattExplorerDisconnectedTask implements Runnable {
        public void run() {
            connStatusTV.setText(R.string.ge_disconnected);
            actionButton.setText(R.string.btn_action);
        }

    }

    class GattExplorerDiscoveredTask implements Runnable {
        public void run() {
            connStatusTV.setText(R.string.ge_discovered);
            actionButton.setText(R.string.btn_disconnect);
        }

    }

    class KeyReceivedTask implements Runnable {
        public KeyReceivedTask( long v ) {
            this.v = v;
        }

        public void run() {
            String msg = "Key received: "+Long.toHexString(v);
            connStatusTV.setText(msg);
        }

        long v;
    }

    class IRGWServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className,
                                               IBinder boundService ) {
            irgwserviceIF = IRGWServiceIF.Stub.asInterface((IBinder)boundService);
            Log.d( LOG_TAG,"onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
            irgwserviceIF = null;
            Log.d(LOG_TAG, "onServiceDisconnected");
        }
    };

}
